/**
 * @file
 *
 * This file contains a class, <code>BinaryTreeNode</code>, instances
 * of which represent immutable nodes in a binary tree. Arbitrary data
 * may be associated with a node.
 */

#include <stdlib.h>
#include <stdbool.h>
#include "BinaryTreeNode.h"

/**
 * This structure represents the instance data of
 * <code>BinaryTreeNode</code> objects.
 */
struct BinaryTreeNode_s
{
  /**
   * Arbitrary data associated with this instance. May be
   * <code>NULL</code>.
   */
  void* pData;

  /**
   * A pointer to the left node pointed to by this node. May be
   * <code>NULL</code>.
   */
  BinaryTreeNode_t* pLeftNode;

  /**
   * A pointer to the right node pointed to by this node. May be
   * <code>NULL</code>.
   */
  BinaryTreeNode_t* pRightNode;
};

bool BinaryTreeNode_Create(void* pData,
                           BinaryTreeNode_t* pLeftNode,
                           BinaryTreeNode_t* pRightNode,
                           BinaryTreeNode_t** ppThis)
{
  BinaryTreeNode_t* pBinaryTreeNodeInstance = NULL;
  bool result = ppThis != NULL;

  if (result)
  {
    pBinaryTreeNodeInstance = malloc(sizeof(BinaryTreeNode_t));
    result = pBinaryTreeNodeInstance != NULL;
  }

  if (result)
  {
    pBinaryTreeNodeInstance->pData = pData;
    pBinaryTreeNodeInstance->pLeftNode = pLeftNode;
    pBinaryTreeNodeInstance->pRightNode = pRightNode;
    *ppThis = pBinaryTreeNodeInstance;
  }
  else if (ppThis != NULL)
  {
    *ppThis = NULL;
  }

  return result;
}

void BinaryTreeNode_Destroy(BinaryTreeNode_t* pThis)
{
  if (pThis != NULL)
  {
    free(pThis);
  }
}

bool BinaryTreeNode_Data(BinaryTreeNode_t* pThis,
                         void** ppData)
{
  bool result = (pThis != NULL) && (ppData != NULL);

  if (result)
  {
    *ppData = pThis->pData;
  }
  else if (ppData != NULL)
  {
    *ppData = NULL;
  }

  return result;
}

bool BinaryTreeNode_LeftNode(BinaryTreeNode_t* pThis,
                             BinaryTreeNode_t** ppLeftNode)
{
  bool result = (pThis != NULL) && (ppLeftNode != NULL);

  if (result)
  {
    *ppLeftNode = pThis->pLeftNode;
  }
  else if (ppLeftNode != NULL)
  {
    *ppLeftNode = NULL;
  }

  return result;
}

bool BinaryTreeNode_RightNode(BinaryTreeNode_t* pThis,
                              BinaryTreeNode_t** ppRightNode)
{
  bool result = (pThis != NULL) && (ppRightNode != NULL);

  if (result)
  {
    *ppRightNode = pThis->pRightNode;
  }
  else if (ppRightNode != NULL)
  {
    *ppRightNode = NULL;
  }

  return result;
}
