#include <stdlib.h>
#include <stdio.h>
#include "BinaryTreeNode.h"

// Global constants.
const unsigned int ONE = 1;
const unsigned int TWO = 2;
const unsigned int THREE = 3;

bool createData(BinaryTreeNode_t** ppRootNode)
{
  bool result = ppRootNode != NULL;
  BinaryTreeNode_t* pRootNode = NULL;
  BinaryTreeNode_t* pRootNodeLeftNode = NULL;
  BinaryTreeNode_t* pRootNodeRightNode = NULL;

  result = BinaryTreeNode_Create((void*)&TWO,
                                 NULL,
                                 NULL,
                                 &pRootNodeLeftNode);

  if (result)
  {
    result = BinaryTreeNode_Create((void*)&THREE,
                                   NULL,
                                   NULL,
                                   &pRootNodeRightNode);
  }

  if (result)
  {
    result = BinaryTreeNode_Create((void*)&ONE,
                                   pRootNodeLeftNode,
                                   pRootNodeRightNode,
                                   &pRootNode);
  }

  if (result)
  {
    *ppRootNode = pRootNode;
  }

  if (!result)
  {
    if (ppRootNode != NULL)
    {
      *ppRootNode = NULL;
    }

    BinaryTreeNode_Destroy(pRootNode);
    BinaryTreeNode_Destroy(pRootNodeRightNode);
    BinaryTreeNode_Destroy(pRootNodeLeftNode);
  }

  return result;
}

bool destroyChildren(BinaryTreeNode_t* pRootNode)
{
  BinaryTreeNode_t* pLeftNode = NULL;
  BinaryTreeNode_t* pRightNode = NULL;

  bool result = pRootNode != NULL;

  if (result)
  {
    result = BinaryTreeNode_LeftNode(pRootNode, &pLeftNode);
  }

  if (result)
  {
    result = BinaryTreeNode_RightNode(pRootNode, &pRightNode);
  }

  if (result)
  {
    BinaryTreeNode_Destroy(pRightNode);
    BinaryTreeNode_Destroy(pLeftNode);
  }

  return result;
}
  
int main()
{
  bool result = false;
  BinaryTreeNode_t* pRootNode = NULL;
  BinaryTreeNode_t* pRootNodeLeftNode = NULL;
  BinaryTreeNode_t* pRootNodeRightNode = NULL;

  unsigned int* rootNodeData = NULL;
  unsigned int* rootNodeLeftNodeData = NULL;
  unsigned int* rootNodeRightNodeData = NULL;

  result = createData(&pRootNode);

  if (result)
  {
    result = BinaryTreeNode_LeftNode(pRootNode,
                                     &pRootNodeLeftNode);
  }

  if (result)
  {
    result = BinaryTreeNode_RightNode(pRootNode,
                                      &pRootNodeRightNode);
  }

  if (result)
  {
    result = BinaryTreeNode_Data(pRootNode,
                                 (void*)&rootNodeData);
  }

  if (result)
  {
    result = BinaryTreeNode_Data(pRootNodeLeftNode,
                                 (void*)&rootNodeLeftNodeData);
  }

  if (result)
  {
    result = BinaryTreeNode_Data(pRootNodeRightNode,
                                 (void*)&rootNodeRightNodeData);
  }

  if (result)
  {
    printf("Root node, data: %d\n", *rootNodeData);
    printf("Left node, data: %d\n", *rootNodeLeftNodeData);
    printf("Right node, data: %d\n", *rootNodeRightNodeData);
  }

  /* Ignore the value of the result variable and destroy the memory
   * allocated regardless of its value. This is, in other words, a
   * "finally" section.
   */
  bool finallyResult = destroyChildren(pRootNode);
  
  if (finallyResult)
  {
    BinaryTreeNode_Destroy(pRootNode);
  }

  return (result && finallyResult) ? 0 : 1;
}
