/**
 * Instances of this class represent immutable nodes in a binary
 * tree. Arbitrary data may be associated with a node.
 *
 * @param <D> 
 *   the type of data objects maintained by this node and its
 *   children.
 */
public final class BinaryTreeNode<D>
{
  /**
   * The client-defined data associated with this node, or {@code
   * null} if there is no such data.
   */
  private final D data;

  /**
   * The left node of this node, or {@code null} if there is no such
   * node.
   */
  private final BinaryTreeNode<D> leftNode;

  /**
   * The right node of this node, or {@code null} if there is no such
   * node.
   */
  private final BinaryTreeNode<D> rightNode;

  /**
   * Creates an instance of this class.
   *
   * @param data
   *   the client-defined data of this node, or {@code null}.
   * @param leftNode
   *   the left node of this node, or {@code null}.
   * @param rightNode
   *   the right node of this node, or {@code null}.
   */
  public BinaryTreeNode(D data, 
                        BinaryTreeNode<D> leftNode,
                        BinaryTreeNode<D> rightNode)
  {
    this.data = data;
    this.leftNode = leftNode;
    this.rightNode = rightNode;
  }

  /**
   * Returns the client-defined data associated with this node, or
   * {@code null} if there is no such data.
   *
   * @return
   *   the data associated with this node, or {@code null}.
   */
  public D data()
  {
    return data;
  }

  /**
   * Returns the left node of this node, or {@code null} if there is
   * no such node.
   *
   * @return
   *   the left node of this node, or {@code null}.
   */
  public BinaryTreeNode<D> leftNode()
  {
    return leftNode;
  }

  /**
   * Returns the right node of this node, or {@code null} if there is
   * no such node.
   *
   * @return
   *   the right node of this node, or {@code null}.
   */
  public BinaryTreeNode<D> rightNode()
  {
    return rightNode;
  }
}
