/**
 * @file
 *
 * This file contains a class,
 * <code>DefaultIdentifierOperandNode</code>, which is the default
 * implementation of the <code>IdentifierOperandNode</code> interface.
 */

#include <stdbool.h>
#include <stdlib.h>
#include <wchar.h>
#include "Node.h"
#include "IdentifierOperandNode.h"
#include "DefaultIdentifierOperandNode.h"

/**
 * This structure represents the instance data of
 * <code>DefaultIdentifierOperandNode</code> objects.
 */
typedef struct DefaultIdentifierOperandNode_InstanceData_s
{
  /**
   * The dispatch table used by this instance.
   */
  const IdentifierOperandNode_DispatchTable_t* pDispatchTable;

  /**
   * The identifier character of this node.
   */
  wchar_t character;
} DefaultIdentifierOperandNode_InstanceData_t;

static const IdentifierOperandNode_DispatchTable_t
  gIdentifierOperandNodeDispatchTable;

bool DefaultIdentifierOperandNode_Create(
  wchar_t character,
  IdentifierOperandNode_t** ppResult)
{
  DefaultIdentifierOperandNode_InstanceData_t* pInstanceData = NULL;

  bool result = (ppResult != NULL);

  if (result)
  {
    pInstanceData =
      malloc(sizeof(DefaultIdentifierOperandNode_InstanceData_t));
    result = pInstanceData != NULL;
  }

  if (result)
  {
    pInstanceData->pDispatchTable =
      &gIdentifierOperandNodeDispatchTable;
    pInstanceData->character = character;
    *ppResult = (IdentifierOperandNode_t*)pInstanceData;
  }
  else if (ppResult != NULL)
  {
    *ppResult = NULL;
  }

  return result;
}

static void DefaultIdentifierOperandNode_Destroy(
  IdentifierOperandNode_t* pIdentifierOperandNode)
{
  if (pIdentifierOperandNode != NULL)
  {
    free(pIdentifierOperandNode);
  }
}

static bool DefaultIdentifierOperandNode_IsConstant(
  IdentifierOperandNode_t* pIdentifierOperandNode, bool* pResult)
{
  bool result = (pIdentifierOperandNode != NULL) && (pResult != NULL);

  if (pResult != NULL)
  {
    *pResult = false;
  }

  return result;
}

static bool DefaultIdentifierOperandNode_IdentifierCharacter(
  IdentifierOperandNode_t* pIdentifierOperandNode,
  wchar_t* pResult)
{
  bool result = (pIdentifierOperandNode != NULL) && (pResult != NULL);
  DefaultIdentifierOperandNode_InstanceData_t* pThis =
    (DefaultIdentifierOperandNode_InstanceData_t*)
    pIdentifierOperandNode;

  if (result)
  {
    *pResult = pThis->character;
  }
  else if (pResult != NULL)
  {
    *pResult = L'\0';
  }

  return result;
}

/**
 * This is the dispatch table for the <code>IdentifierOperandNode</code>
 * interface implemented by this class.
 */
static const IdentifierOperandNode_DispatchTable_t
  gIdentifierOperandNodeDispatchTable =
{
  DefaultIdentifierOperandNode_Destroy,
  DefaultIdentifierOperandNode_IsConstant,
  DefaultIdentifierOperandNode_IdentifierCharacter
};
