/**
 * @file
 *
 * This file contains a class, <code>DefaultLiteralOperandNode</code>,
 * which is the default implementation of the
 * <code>LiteralOperandNode</code> interface.
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdarg.h>
#include "Node.h"
#include "LiteralOperandNode.h"
#include "DefaultLiteralOperandNode.h"

/**
 * This structure represents the instance data of
 * <code>DefaultLiteralOperandNode</code> objects.
 */
typedef struct DefaultLiteralOperandNode_InstanceData_s
{
  /**
   * The dispatch table used by this instance.
   */
  const LiteralOperandNode_DispatchTable_t* pDispatchTable;

  /**
   * The constant of this node.
   */
  double constant;
} DefaultLiteralOperandNode_InstanceData_t;

static const LiteralOperandNode_DispatchTable_t
  gLiteralOperandNodeDispatchTable;

bool DefaultLiteralOperandNode_Create(double constant,
                                      LiteralOperandNode_t** ppResult)
{
  DefaultLiteralOperandNode_InstanceData_t* pInstanceData = NULL;

  bool result = (ppResult != NULL);

  if (result)
  {
    pInstanceData =
      malloc(sizeof(DefaultLiteralOperandNode_InstanceData_t));
    result = pInstanceData != NULL;
  }

  if (result)
  {
    pInstanceData->pDispatchTable = &gLiteralOperandNodeDispatchTable;
    pInstanceData->constant = constant;
    *ppResult = (LiteralOperandNode_t*)pInstanceData;
  }
  else if (ppResult != NULL)
  {
    *ppResult = NULL;
  }

  return result;
}

static void DefaultLiteralOperandNode_Destroy(
  LiteralOperandNode_t* pLiteralOperandNode)
{
  if (pLiteralOperandNode != NULL)
  {
    free(pLiteralOperandNode);
  }
}

static bool DefaultLiteralOperandNode_IsConstant(
  LiteralOperandNode_t* pLiteralOperandNode, bool* pResult)
{
  bool result = (pLiteralOperandNode != NULL) && (pResult != NULL);

  if (result)
  {
    *pResult = true;
  }
  else if (pResult != NULL)
  {
    *pResult = false;
  }

  return result;
}

static bool DefaultLiteralOperandNode_Constant(
  LiteralOperandNode_t* pLiteralOperandNode, double* pResult)
{
  bool result = (pLiteralOperandNode != NULL) && (pResult != NULL);
  DefaultLiteralOperandNode_InstanceData_t* pThis =
    (DefaultLiteralOperandNode_InstanceData_t*)pLiteralOperandNode;

  if (result)
  {
    *pResult = pThis->constant;
  }
  else if (pResult != NULL)
  {
    *pResult = 0.0;
  }

  return result;
}

/**
 * This is the dispatch table for the <code>LiteralOperandNode</code>
 * interface implemented by this class.
 */
static const LiteralOperandNode_DispatchTable_t
  gLiteralOperandNodeDispatchTable =
{
  DefaultLiteralOperandNode_Destroy,
  DefaultLiteralOperandNode_IsConstant,
  DefaultLiteralOperandNode_Constant
};
