/**
 * @file
 *
 * This file contains function prototypes for the class operations
 * (static operations) of the <code>DefaultUnaryOperatorNode</code>
 * class.
 */

#ifndef INCLUSION_GUARD_DEFAULT_UNARY_OPERATOR_NODE
#define INCLUSION_GUARD_DEFAULT_UNARY_OPERATOR_NODE

#include <stdbool.h>
#include "Fundamental.h"
#include "Node.h"
#include "UnaryOperatorNode.h"
#include "UnaryOperator.h"

/**
 * Creates an instance of the <code>DefaultUnaryOperatorNode</code>
 * class.
 *
 * @param[in] operator
 *   the unary operator of this node.
 * @param[in] pOperand
 *   the sole operand of this node. Must not be <code>NULL</code>.
 * @param[in] pInterfaceName
 *   the name of the interface that the reference should be returned
 *   through. Must not be <code>NULL</code>.
 * @param[out] ppResult
 *   a pointer to the variable which shall hold the instance of this
 *   class. Must not be <code>NULL</code>.
 * @return
 *   <code>true</code> if the operation completes successfully,
 *   <code>false</code> otherwise.
 */
bool DefaultUnaryOperatorNode_Create(UnaryOperator_t operator,
                                     Node_t* pOperand,
                                     wchar_t* pInterfaceName,
                                     Fundamental_t** ppResult);

#endif // INCLUSION_GUARD_DEFAULT_UNARY_OPERATOR_NODE
