/**
 * @file
 *
 * This file contains an enumerated type representing the different
 * types that may be passed to (and returned from) the
 * <code>Scriptable::InvokeOperation()</code> operation.
 */

#ifndef INCLUSION_GUARD_SCRIPTABLE_ARGUMENT_TYPE
#define INCLUSION_GUARD_SCRIPTABLE_ARGUMENT_TYPE

/**
 * This enumerated type represents the different types that may be
 * passed to (and returned from) the
 * <code>Scriptable::InvokeOperation()</code> operation.
 */
typedef enum
{
  /**
   * The type is invalid.
   */
  ScriptableArgumentType_INVALID,

  /**
   * There is no type. This is used for non-existent return values,
   * and may not be used for arguments to an operation.
   */
  ScriptableArgumentType_VOID,

  /**
   * The type represents an unsigned integer (an <code>unsigned
   * int</code>).
   */
  ScriptableArgumentType_INTEGER,

  /**
   * The type represents a double-precision floating-point value
   * conforming to IEEE 754 (a <code>double</code>).
   */
  ScriptableArgumentType_DOUBLE,

  /**
   * The type represents a boolean value (a <code>bool</code>).
   */
  ScriptableArgumentType_BOOLEAN,

  /**
   * The type represents a character (a <code>wchar_t</code>).
   */
  ScriptableArgumentType_CHARACTER,

  /**
   * The type represents an arbitrary object which implements the
   * <code>Scriptable</code> interface.
   */
  ScriptableArgumentType_SCRIPTABLE,

  /**
   * This enumerator signifies that no more arguments are expected in
   * a call to <code>Scriptable::InvokeOperation()</code>.
   */
  ScriptableArgumentType_NO_MORE_ARGUMENTS
} ScriptableArgumentType_t;

#endif // INCLUSION_GUARD_SCRIPTABLE_ARGUMENT_TYPE
