/**
 * A binary arithmetic operator, such as addition or multiplication.
 */
public enum BinaryOperator {
  ADDITION       ('+'), /** Addition (binary plus) ("2+x"). */
  SUBTRACTION    ('-'), /** Subtraction (binary minus) ("x-4"). */
  MULTIPLICATION ('*'), /** Multiplication ("x*d"). */
  DIVISION       ('/'), /** Division ("a/(1/3)"). */
  ASSIGNMENT     ('='); /** Assignment ("a = 3"). */

  /**
   * The symbol used for this operator.
   */
  private char symbol;

  /**
   * Constructs an instance of this class, with the given symbol.
   *
   * @param symbol
   *   the symbol of this operator.
   */
  BinaryOperator(char symbol) {
    this.symbol = symbol;
  }

  /**
   * Returns the symbol used for this operator.
   *
   * @return
   *   the symbol used for this operator.
   */
  public char symbol() {
    return symbol;
  }

  /**
   * Returns a concise string representation of this operator.
   *
   * @return
   *   a concise string representation of this operator.
   */
  public String toConciseString() {
    return new String(new char[] { symbol } );
  }
}
