/**
 * This interface represents a binary operator node in a syntax tree
 * representing an arithmetic expression. A binary operator takes two
 * operands (in the expression "3 / x", the division operator operates
 * on the literal operand 3 and the identifier operator "x").
 */
public interface BinaryOperatorNode extends Node {
  /**
   * Returns the binary operator of this node.
   *
   * @return
   *   the binary operator of this node.
   */
  public BinaryOperator operator();

  /**
   * Returns the left operand of this node.
   *
   * @return
   *   the left operand of this node.
   */
  public Node leftOperand();

  /**
   * Returns the right operand of this node.
   *
   * @return
   *   the right operand of this node.
   */
  public Node rightOperand();
}
