/**
 * Instances of this class represent binary operators (such as
 * division, as in "3 / x").
 */
final class DefaultBinaryOperatorNode implements BinaryOperatorNode {
  /**
   * The binary operator of this node.
   */
  private final BinaryOperator operator;
  
  /**
   * The first operand.
   */
  private Node leftOperand;

  /**
   * The second operand.
   */
  private Node rightOperand;
  
  /**
   * Creates an instance of this node. No parameters may be {@code
   * null}.
   *
   * @param operator
   *   the operator of this node.
   * @param leftOperand
   *   the first operand.
   * @param rightOperand
   *   the second operand.
   * @throws InvalidArgumentException
   *   if any of the parameters are {@code null}.
   */
  public DefaultBinaryOperatorNode(BinaryOperator operator,
                                   Node leftOperand,
                                   Node rightOperand) {
    if ((operator == null) || 
        (leftOperand == null) || 
        (rightOperand == null))
      throw new IllegalArgumentException("No parameters may be null.");

    this.operator = operator;
    this.leftOperand = leftOperand;
    this.rightOperand = rightOperand;
  }

  /**
   * Returns the operator of this node.
   *
   * @return
   *   the operator of this node.
   */
  public BinaryOperator operator() {
    return operator;
  }

  /**
   * Returns the left operand of this node.
   *
   * @return
   *   the left operand of this node.
   */
  public Node leftOperand() {
    return leftOperand;
  }

  /**
   * Returns the right operand of this node.
   *
   * @return
   *   the right operand of this node.
   */
  public Node rightOperand() {
    return rightOperand;
  }

  public boolean isConstant() {
    return leftOperand.isConstant() && rightOperand.isConstant();
  }
}
