/**
 * Instances of this class represent unary operators (such as
 * negation, as in "3 + -2").
 */
final class DefaultUnaryOperatorNode implements UnaryOperatorNode {
  /**
   * The unary operator of this node.
   */
  private UnaryOperator operator;

  /**
   * The operand to be operated on by the operator.
   */
  private Node operand;
  
  /**
   * Creates an instance of this node. No parameters may be {@code
   * null}.
   *
   * @param operator
   *   the operator of this node.
   * @param operand
   *   the operand operated on by the operator.
   * @throws IllegalArgumentException
   *   if any of the parameters are {@code null}.
   */
  public DefaultUnaryOperatorNode(UnaryOperator operator, 
                                  Node operand) {
    if ((operator == null) || (operand == null))
      throw new IllegalArgumentException("Parameters must not be null.");

    this.operator = operator;
    this.operand = operand;
  }

  /**
   * Returns the operator of this node.
   *
   * @return
   *   the operator of this node.
   */
  public UnaryOperator operator() {
    return operator;
  }

  public boolean isConstant() {
    return operand.isConstant();
  }

  /**
   * Returns the operand of this node.
   *
   * @return
   *   the operand of this node.
   */
  public Node operand() {
    return operand;
  }
}
