/**
 * This interface represents an immutable node in a syntax tree
 * representing an arithmetic expression (as produced by an
 * operator-precedence parser). This interface is not meant to be
 * implemented directly; rather, it stipulates what functionality all
 * nodes in said tree must be able to provide. Descendant interfaces
 * must add operations specific to the type of node they are modeling.
 */
public interface Node {
  /**
   * Returns {@code true} if this node is fully constant, {@code
   * false} otherwise. Nodes which are fully constant may be
   * collapsed, meaning that they may be replaced with a literal node
   * representing the subtree of the node.
   *
   * @return
   *   whether this node is constant (that is, there are only literal
   *   operands as part of the subtree represented by the node).
   */
  public boolean isConstant();
}
