/**
 * A unary arithmetic operator, such as negation.
 */
public enum UnaryOperator {
  PLUS  ('+'), /** Unary plus, does not change the operand ("2 - +3"). */
  MINUS ('-'); /** Unary minus, negates the operand ("2 - -3"). */

  /**
   * The symbol used for this operator.
   */
  private char symbol;

  /**
   * Constructs an instance of this class, with the given symbol.
   *
   * @param symbol
   *   the symbol of this operator.
   */
  UnaryOperator(char symbol) {
    this.symbol = symbol;
  }

  /**
   * Returns the symbol used for this operator.
   *
   * @return
   *   the symbol used for this operator.
   */
  public char symbol() {
    return symbol;
  }

  /**
   * Returns a concise string representation of this operator.
   *
   * @return
   *   a concise string representation of this operator.
   */
  public String toConciseString() {
    char[] symbolCharArray;

    /* If this instance represents a unary minus, use an underscore
     * instead of a minus, to make it possible to differentiate it
     * from subtraction.
     */
    if (this == MINUS)
      symbolCharArray = new char[] { '_' };
    else 
      symbolCharArray = new char[] { symbol };

    return new String(symbolCharArray);
  }
}
